#define CUSTOM_SETTINGS
#define INCLUDE_GAMEPAD_MODULE
#include <Dabble.h>
#include <Servo.h>

//Servo Connections
Servo base;
int basepin = 13;
int basepos = 90;

Servo arm1;
int arm1pin = 12;
int arm1pos = 180;

Servo arm2;
int arm2pin = 11;
int arm2pos = 180;

Servo arm3;
int arm3pin = 8;
int arm3pos = 90;

Servo arm4;
int arm4pin = 9;
int arm4pos = 0;

Servo arm5;
int arm5pin = 10;
int arm5pos = 0;

// Motor A connections
int enA = 2;
int in1 = 3;
int in2 = 4;
// Motor B connections
int enB = 7;
int in3 = 5;
int in4 = 6;

//Variables for controlling the servos
int activecontrol = 1;
int servoDelay = 10; //Adjust this as needed


void setup() {
  Serial.begin(9600);  // Make sure this is the same as the bluetooth baudrate.
  Dabble.begin(9600);    //Baudrate of the HM-10 bluetooth module.
  
  //Setup Servo Pins
  base.attach(basepin);
  arm1.attach(arm1pin);
  arm2.attach(arm2pin);
  arm3.attach(arm3pin); 
  arm4.attach(arm4pin);
  arm5.attach(arm5pin);


  // Set all the motor control pins to outputs
  pinMode(enA, OUTPUT);
  pinMode(enB, OUTPUT);
  pinMode(in1, OUTPUT);
  pinMode(in2, OUTPUT);
  pinMode(in3, OUTPUT);
  pinMode(in4, OUTPUT);

  // Turn off motors - Begin state
  digitalWrite(in1, LOW);
  digitalWrite(in2, LOW);
  digitalWrite(in3, LOW);
  digitalWrite(in4, LOW);

  //Set all servos to default position
  base.write(90);
  arm1.write(180);
  arm2.write(180);
  arm3.write(90);
  arm4.write(0);
  arm5.write(0);
}

void controlMotorWithJoystick() {
  float x = GamePad.getXaxisData(); // Joystick X-axis value (-7 to 7)
  float y = GamePad.getYaxisData(); // Joystick Y-axis value (-7 to 7)

  // Calculate motor speeds
  int leftSpeed = constrain((y - x) * 36, -255, 255);  // Swapped: subtract x for left motor
  int rightSpeed = constrain((y + x) * 36, -255, 255); // Swapped: add x for right motor

  // Set motor A (left side)
  if (leftSpeed > 0) {
    analogWrite(enA, leftSpeed);
    digitalWrite(in1, HIGH);
    digitalWrite(in2, LOW);
  } else if (leftSpeed < 0) {
    analogWrite(enA, abs(leftSpeed));
    digitalWrite(in1, LOW);
    digitalWrite(in2, HIGH);
  } else {
    digitalWrite(in1, LOW);
    digitalWrite(in2, LOW);
  }

  // Set motor B (right side)
  if (rightSpeed > 0) {
    analogWrite(enB, rightSpeed);
    digitalWrite(in3, HIGH);
    digitalWrite(in4, LOW);
  } else if (rightSpeed < 0) {
    analogWrite(enB, abs(rightSpeed));
    digitalWrite(in3, LOW);
    digitalWrite(in4, HIGH);
  } else {
    digitalWrite(in3, LOW);
    digitalWrite(in4, LOW);
  }
}

void baseleft() {
  if (basepos > 0) {
    basepos += 1;          // Increase the servo position (move left)
    base.write(basepos); // tell servo to go to position in variable 'pos'
    delay(servoDelay);          // short delay to allow servo to reach the position
  }
}

void baseright() {
  if (basepos < 180) {
    basepos -= 1;          // Decrease the servo position (move right)
    base.write(basepos); // tell servo to go to position in variable 'pos'
    delay(servoDelay);          // short delay to allow servo to reach the position
  }
}

void arm1left() {
  if (arm1pos > 0) {
    arm1pos -= 1;          // Decrease the servo position (move left)
    arm1.write(arm1pos);   // Move the servo
    delay(servoDelay);     // Short delay to allow the servo to reach the position
  }
}

void arm1right() {
  if (arm1pos < 180) {
    arm1pos += 1;          // Increase the servo position (move right)
    arm1.write(arm1pos);   // Move the servo
    delay(servoDelay);     // Short delay to allow the servo to reach the position
  }
}

void arm2left() {
  if (arm2pos > 0) {
    arm2pos -= 1;          // decrease the servo position (move left)
    arm2.write(arm2pos); // tell servo to go to position in variable 'pos'
    delay(servoDelay);          // short delay to allow servo to reach the position
  }
}

void arm2right() {
  if (arm2pos < 180) {
    arm2pos += 1;          // increase the servo position (move right)
    arm2.write(arm2pos); // tell servo to go to position in variable 'pos'
    delay(servoDelay);          // short delay to allow servo to reach the position
  }
}

void arm3left() {
  if (arm3pos > 0) {
    arm3pos -= 1;          // decrease the servo position (move left)
    arm3.write(arm3pos); // tell servo to go to position in variable 'pos'
    delay(servoDelay);          // short delay to allow servo to reach the position
  }
}

void arm3right() {
  if (arm3pos < 360) { //This servo is the 360o degree servo.
    arm3pos += 1;          // increase the servo position (move right)
    arm3.write(arm3pos); // tell servo to go to position in variable 'pos'
    delay(servoDelay);          // short delay to allow servo to reach the position
  }
}

void arm4left() {
  if (arm4pos > 0) {
    arm4pos -= 1;          // decrease the servo position (move left)
    arm4.write(arm4pos); // tell servo to go to position in variable 'pos'
    delay(servoDelay);          // short delay to allow servo to reach the position
  }
}

void arm4right() {
  if (arm4pos < 180) {  
    arm4pos += 1;          // increase the servo position (move right)
    arm4.write(arm4pos); // tell servo to go to position in variable 'pos'
    delay(servoDelay);          // short delay to allow servo to reach the position
  }
}
void arm5left() {
  if (arm5pos > 0) {
    arm5pos -= 1;          // decrease the servo position (move left)
    arm5.write(arm5pos); // tell servo to go to position in variable 'pos'
    delay(servoDelay);          // short delay to allow servo to reach the position
  }
}

void arm5right() {
  if (arm5pos < 180) {
    arm5pos += 1;          // increase the servo position (move right)
    arm5.write(arm5pos); // tell servo to go to position in variable 'pos'
    delay(servoDelay);          // short delay to allow servo to reach the position
  }
}



void forward()  { 

  analogWrite(enA, 100);
  analogWrite(enB, 100);

  // Turn on motor A & B
  digitalWrite(in1, HIGH);
  digitalWrite(in2, LOW);
  digitalWrite(in3, HIGH);
  digitalWrite(in4, LOW);
  delay(100);

  // Turn off motors  
  digitalWrite(in1, LOW);
  digitalWrite(in2, LOW);
  digitalWrite(in3, LOW);
  digitalWrite(in4, LOW);
}

void backward() {

  analogWrite(enA, 100);
  analogWrite(enB, 100);

  // Turn on motor A & B
  digitalWrite(in1, LOW);
  digitalWrite(in2, HIGH);
  digitalWrite(in3, LOW);
  digitalWrite(in4, HIGH);
  delay(100);

  // Turn off motors  
  digitalWrite(in1, LOW);
  digitalWrite(in2, LOW);
  digitalWrite(in3, LOW);
  digitalWrite(in4, LOW);
}

void left() {
  analogWrite(enA, 100);
  analogWrite(enB, 100);

  // Motor A forward, Motor B backward
  digitalWrite(in1, HIGH);  // Motor A forward
  digitalWrite(in2, LOW);
  digitalWrite(in3, LOW);  // Motor B backward
  digitalWrite(in4, HIGH);
  delay(100);

  // Turn off motors  
  digitalWrite(in1, LOW);
  digitalWrite(in2, LOW);
  digitalWrite(in3, LOW);
  digitalWrite(in4, LOW);
}

void right() {
  analogWrite(enA, 100);
  analogWrite(enB, 100);

  // Motor A backward, Motor B forward
  digitalWrite(in1, LOW);  // Motor A backward
  digitalWrite(in2, HIGH);
  digitalWrite(in3, HIGH);  // Motor B forward
  digitalWrite(in4, LOW);
  delay(100);

  // Turn off motors  
  digitalWrite(in1, LOW);
  digitalWrite(in2, LOW);
  digitalWrite(in3, LOW);
  digitalWrite(in4, LOW);
}


void loop() {
  Dabble.processInput();  //this function is used to refresh data obtained from smartphone.Hence calling this function is mandatory in order to get data properly from your mobile.

  Serial.print("KeyPressed: ");
  if (GamePad.isUpPressed()) {
    Serial.print("UP");
    forward();
  }

  if (GamePad.isDownPressed()) {
    Serial.print("DOWN");
    backward();
  }

  if (GamePad.isLeftPressed()) {
    Serial.print("Left");
    left();
  }

  if (GamePad.isRightPressed()) {
    Serial.print("Right");
    right();
  }

  if (GamePad.isSquarePressed()) {
    Serial.print("Square");

    if (activecontrol == 1) {
      baseleft();
    }
    else if (activecontrol == 2){
      arm3right();
    }
    else if (activecontrol == 3) {
      arm5left();
    }
  }

  if (GamePad.isCirclePressed()) {
    Serial.print("Circle");

    if (activecontrol == 1) {
      baseright();
    }
    else if (activecontrol == 2){
      arm3left();
    }
    else if (activecontrol == 3){
      arm5right();
    }
  }

  if (GamePad.isCrossPressed()) {
    Serial.print("Cross");
    
    if (activecontrol == 1) {
      arm1left();
    }
    else if (activecontrol == 2){
      arm2left();
    }
    else if (activecontrol == 3){
      arm4left();
    }
  }

  if (GamePad.isTrianglePressed()) {
    Serial.print("Triangle");

    if (activecontrol == 1) {
      arm1right();
    }
    else if (activecontrol == 2){
      arm2right();
    }
    else if (activecontrol == 3){
      arm4right();
    }
  }

if (GamePad.isStartPressed()) {
  activecontrol = (activecontrol == 3) ? 1 : 3;
  delay(100);
}
if (GamePad.isSelectPressed()) {
  activecontrol = (activecontrol == 2) ? 1 : 2;
  delay(100);
}

controlMotorWithJoystick();

  Serial.print('\t');

  int a = GamePad.getAngle();
  Serial.print("Angle: ");
  Serial.print(a);
  Serial.print('\t');
  int b = GamePad.getRadius();
  Serial.print("Radius: ");
  Serial.print(b);
  Serial.print('\t');
  float c = GamePad.getXaxisData();
  Serial.print("x_axis: ");
  Serial.print(c);
  Serial.print('\t');
  float d = GamePad.getYaxisData();
  Serial.print("y_axis: ");
  Serial.println(d);
  Serial.println();
}
